//= require <dSS/GetTextHelper>
//= require <dSS/util/Util>
Ext.namespace('dSS');

dSS.ScenesPanel = Ext.extend(Ext.Panel, {
    initComponent: function() {

        var contentPanel = {
            ref: 'contentPanel',
            region: 'center',
            layout: 'card',
            hideBorders: true,
            border: true,
            activeItem: 0,
            items: [
            {
                xtype: 'dssdevicepanel'
            }]
        };

        this.treePanel = new Ext.tree.TreePanel({
            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            border: false,
            rootVisible: false,
            root: new Ext.tree.AsyncTreeNode(),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.treePanel.getRootNode().expand();

        this.treeHolder = new Ext.Panel({
            title: gt.ngettext("Scene", 2),
            region: 'west',
            layout: 'fit',
            split: true,
            width: 225,
            minSize: 175,
            maxSize: 400,
            tbar: this.buildTopToolbar(),
            items: [ this.treePanel ]
        });

        Ext.apply(this, {
            layout: 'border',
            border: false,
            bodyBorder: false,
            items: [this.treeHolder, contentPanel]
        });

        this.dirtyTree = false;
        this.updateTree();

        dSS.data.Loader.getZoneStore().addListener('datachanged',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getZoneStore().addListener('update',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getSceneStore().addListener('datachanged',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }
                , this);

        dSS.data.Loader.getSceneStore().addListener('update',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.ScenesPanel.superclass.initComponent.apply(this, arguments);

        this.on('activate', this.handleTreeView.createDelegate(this), this);
        this.on('redrawtree', this.handleTreeView.createDelegate(this), this,
                { buffer: 1000 });

        this.treePanel.on('contextmenu', function(node, event) {
            event.preventDefault();
            event.stopEvent();

            if (!node.isSelected()) {
                node.select();
                this.handleNodeClick(node);
            }

            if (!node.isLeaf()) {
                return;
            }

            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var menuItem = new Ext.menu.Item({
                text: _("Call scene"),
                iconCls: 'callSceneIcon',
                handler: this.callScene.createDelegate(this, [node])
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: _("Save scene"),
                iconCls: 'saveScene',
                handler: this.saveScene.createDelegate(this, [node])
            });
            this.contextMenu.add(menuItem);

            var path = node.getPath().split('/').slice(1);
            if (path.length === 4) {
                menuItem = new Ext.menu.Item({
                    text: _("Rename scene"),
                    iconCls: 'editIcon',
                    handler: this.renameSelectedScene.createDelegate(this, [node])
                });
                this.contextMenu.add(menuItem);
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },
    markDirty: function() {
        this.dirtyTree = true;
        if (this.isVisible() === true) {
            this.fireEvent('redrawtree');
        }
    },

    handleTreeView: function() {
        if (this.dirtyTree === true) {
            this.updateTree();
            return;
        }

        var selModel = this.treePanel.getSelectionModel();
        var node = selModel.getSelectedNode();
        var root = this.treePanel.getRootNode();
        if (node === null) {
            if (root.hasChildNodes()) {
                root.firstChild.select();
                this.handleNodeClick(root.firstChild);
            } else {
              this.updateTree();
            }
        } else {
            this.handleNodeClick(node);
        }
    },

    getSceneName: function(zone, group, scene) {
        var sceneStore = dSS.data.Loader.getSceneStore();
        var rec = sceneStore.getById(zone + '-' + group + '-' + scene);
        if (rec === undefined) {
            return '';
        }

        var name = rec.get('name');
        if ((name === undefined) || (name.length === 0)) {
            return '';
        }

        return ' - ' + name;
    },

    generateSceneNode: function(zone, group, scene, icon) {
        var node = {};
        node.text = dSS.config.sceneNames[scene] +
                    this.getSceneName(zone, group, scene);
        node.id = zone + '-' + group + '-' + scene;
        node.leaf = true;
        node.iconCls = icon;
        return node;
    },

    // generates scenes 0-4 for a given zone/group
    generate0to4Scenes: function(zone, group, icon) {
        var addScenes =
        [
            dSS.config.scenes.T0_S0, dSS.config.scenes.T0_S1,
            dSS.config.scenes.T0_S2, dSS.config.scenes.T0_S3,
            dSS.config.scenes.T0_S4
        ];
        var scenes = [];

        for (var i = 0; i < addScenes.length; i++) {
            scenes.push(
                this.generateSceneNode(zone, group, addScenes[i], icon));
        }
        return scenes;
    },

    // for leafs the id is encoded as: zone id - group - scene number
    // when no group is desired, 0 is used
    generateSubNodes: function(zoneId, groups) {
        var retNodes = [];
        if (groups.LIGHT === true) {
            retNodes.push(
            {
                text: _("Light scenes"),
                id: zoneId + '-' + dSS.config.groups.LIGHT + '-all',
                leaf: false,
                children: this.generate0to4Scenes(zoneId, 
                                                  dSS.config.groups.LIGHT,
                                                  'lightbulbOn')
            });
        }

        if (groups.SHADOW === true) {
            retNodes.push({
                text: _("Shade scenes"),
                id: zoneId + '-' + dSS.config.groups.SHADOW + '-all',
                leaf: false,
                children: this.generate0to4Scenes(zoneId,
                                                  dSS.config.groups.SHADOW,
                                                  'shadesIcon')
            });
        }

        if (groups.ACCESS === true) {
            retNodes.push({
                text: _("Access scenes"),
                id: zoneId + '-' + dSS.config.groups.ACCESS + '-all',
                leaf: false,
                children: this.generate0to4Scenes(zoneId,
                                                  dSS.config.groups.ACCESS,
                                                  'accessIcon')
            });
        }

        if (groups.SECURITY === true) {
            retNodes.push({
                text: _("Security scenes"),
                id: zoneId + '-' + dSS.config.groups.SECURITY + '-all',
                leaf: false,
                children: this.generate0to4Scenes(zoneId,
                                                  dSS.config.groups.SECURITY,
                                                  'securityIcon')
            });
        }

        return retNodes;
    },

    genererateApartmentNodes: function() {
        return [
            {
                text: _("Leave home"),
                id: 0 + '-all-' + dSS.config.scenes.ABSENT_SHORT,
                leaf: true,
                iconCls: 'leaveHomeIcon'
            },
            {
                text: _("Coming home"),
                id: 0 + '-all-' + dSS.config.scenes.PRESENT_SHORT,
                leaf: true,
                iconCls: 'comeHomeIcon'
            },
            {
                text: _("Standby"),
                id: 0 + '-all-' + dSS.config.scenes.STANDBY_SHORT,
                leaf: true,
                iconCls: 'standByIcon'
            },
            {
                text: _("Deep off"),
                id: 0 + '-all-' + dSS.config.scenes.DEEP_OFF_SHORT,
                leaf: true,
                iconCls: 'deepOffIcon'
            },
            {
                text: _("Sleeping"),
                id: 0 + '-all-' + dSS.config.scenes.SLEEPING_SHORT,
                leaf: true,
                iconCls: 'sleepingIcon'
            },
            {
                text: _("Wake up"),
                id: 0 + '-all-' + dSS.config.scenes.WAKE_UP_SHORT,
                leaf: true,
                iconCls: 'wakeupIcon'
            },
            {
                text: _("Door bell"),
                id: 0 + '-all-' + dSS.config.scenes.SIG_BELL_SHORT,
                leaf: true,
                iconCls: 'doorBellIcon'
            },
            {
                text: _("Panic"),
                id: 0 + '-all-' + dSS.config.scenes.PANIC_SHORT,
                iconCls: 'panicIcon',
                leaf: true
            }
        ];
    },

    updateTree: function() {
        var devStore = dSS.data.Loader.getDeviceStore();
        devStore.clearFilter();


        var selModel = this.treePanel.getSelectionModel();
        var selectedNode = selModel.getSelectedNode();
        var selectedPath = '';
        if (selectedNode !== null) {
            selectedPath = selectedNode.getPath();
        }

        var zStore = dSS.data.Loader.getZoneStore();
        var nodes = [];
        zStore.each(
        function(record) {
            if ((record.get('id') !== 0) && (devStore.find('zoneId', record.get('id')) >= 0)) {
                var deviceTypesInZone = {};
                var addZone = false;
                devStore.each(function(device) {
                    if (device.get('zoneId') == record.get('id')) {
                        var fid = dSS.util.parseFunctionID(device.get('functionID'));
                        switch (fid[0]) {
                            case dSS.config.groups.LIGHT:
                               deviceTypesInZone.LIGHT = true;
                               addZone = true;
                               break;
                            case dSS.config.groups.SHADOW:
                                deviceTypesInZone.SHADOW = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.CLIMATE:
                                deviceTypesInZone.CLIMATE = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.AUDIO:
                                deviceTypesInZone.AUDIO = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.VIDEO:
                                deviceTypesInZone.VIDEO = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.SECURITY:
                                deviceTypesInZone.SECURITY = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.ACCESS:
                                deviceTypesInZone.ACCESS = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.JOKER:
                                deviceTypesInZone.JOKER = true;
                               addZone = true;
                                break;
                            default:
                                break;
                        }
                    }
                });
                if (addZone === true) {
                    nodes.push({
                        text: record.get('name') + " - " + record.get('id'),
                        id: record.get('id'),
                        leaf: false,
                        children: this.generateSubNodes(record.get('id'),
                                                        deviceTypesInZone)
                    });
                }
            }
            return true;
        },
        this);

        var root = new Ext.tree.AsyncTreeNode({
            children: nodes.concat(this.genererateApartmentNodes()),
            id: 'root',
            leaf: false
        });
        this.treePanel.setRootNode(root);

        if (selectedPath.length > 0) {
            this.treePanel.selectPath(selectedPath, null, function(result, node) {
                if (result === false) {
                    if (root.firstChild !== null) {
                        root.firstChild.select();
                    }
                }
            });
        } else {
            if (root.firstChild !== null) {
                root.firstChild.select();
            }
        }

        if (selModel.getSelectedNode() !== null) {
            this.handleNodeClick(selModel.getSelectedNode());
        }

        this.dirtyTree = false;
    },

    handleNodeClick: function(node) {
        if (!this.isVisible()) {
            return;
        }

        if (node === null) {
            return;
        }

        if (node.isLeaf() === true) {
            this.treeHolder.getTopToolbar().get('callSceneButton').enable();
            this.treeHolder.getTopToolbar().get('saveSceneButton').enable();
        } else {
            this.treeHolder.getTopToolbar().get('callSceneButton').disable();
            this.treeHolder.getTopToolbar().get('saveSceneButton').disable();
        }

        // we are on the zone node
        var path = node.getPath().split('/').slice(1);
        if (path.length === 4) {
            this.treeHolder.getTopToolbar().get('renameSceneButton').enable();
        } else {
            this.treeHolder.getTopToolbar().get('renameSceneButton').disable();
        }

        if (path.length < 2) {
            return;
        }

        var store = dSS.data.Loader.getDeviceStore();
        store.clearFilter();
        store.filterBy(function(record) {
            var zoneId = parseInt(path[1], 10);

            if (zoneId === 0) {
                return true;
            }

            if (record.get('zoneId') === zoneId) {
                if (path.length === 2) {
                    return true;
                }

                var fid = dSS.util.parseFunctionID(record.get('functionID'));

                var nodeType = path[2].split('-');
                // all leaf node ids will have 3 components: zone-group-scene
                if (nodeType.length < 3) {
                    return false;
                }

                // if the group is set, we sort only by group
                if (nodeType[1] !== 'all') {
                    var group = parseInt(nodeType[1], 10);
                    if (!isNaN(group)) {
                        if (fid[0] === group) {
                            return true;
                        }

                        return false;
                    }
                }

                // at this stage groups do not matter, scene filtering applies
                var scene = parseInt(nodeType[2], 10);
                if (isNaN(scene)) {
                    return false;
                }
            }
            return false;
        });
    },
    buildTopToolbar: function() {
        return [
            {
                iconCls: 'saveScene',
                handler: this.saveSelectedScene.createDelegate(this),
                id: 'saveSceneButton',
                tooltip: _("Save selected scene"),
                disabled: true,
                scope: this
            },
            {
                iconCls: 'editIcon',
                handler: this.renameSelectedScene.createDelegate(this),
                tooltip: _("Rename selected scene"),
                id: 'renameSceneButton',
                disabled: true,
                scope: this
            },
            {
                iconCls: 'callSceneIcon',
                handler: this.callSelectedScene.createDelegate(this),
                id: 'callSceneButton',
                tooltip: _("Call selected scene"),
                disabled: true,
                scope: this
            },
            '->',
            {
                cls: 'zonePanelMenu',
                menu: [
                {
                    text: _("Reload"),
                    handler: this.reload,
                    scope: this
                },
                {
                    text: _("Rescan"),
                    handler: this.rescan,
                    scope: this
                }]
            }];
    },
    rescan: function() {
        this.ownerCt.zoneBrowser.rescan();
    },
    reload: function() {
        this.ownerCt.zoneBrowser.loadData();
    },

    getZoneSceneGroup: function(id) {
        var ret = {
            "zoneId": null,
            "sceneNumber": null,
            "groupId": null
        };

        var ids = id.split('-');
        if (ids.length < 3) {
            return ret;
        }

        var zoneId = parseInt(ids[0], 10);
        if (isNaN(zoneId)) {
            return ret;
        }

        ret.zoneId = zoneId;

        var groupId = parseInt(ids[1], 10);
        if (isNaN(groupId)) {
            groupId = null;
        }

        ret.groupId = groupId;

        var sceneNumber = parseInt(ids[2], 10);
        if (isNaN(sceneNumber)) {
            sceneNumber = null;
        }

        ret.sceneNumber = sceneNumber;

        return ret;
    },

    callSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(_("Error"), _("No scene selected, please select the scene that you would like to call"));
            return;
        }

        this.callScene(node);
    },

    callScene: function(node) {
        if ((node === null) || (!node.isLeaf())) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        dSS.util.callSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
    },
    saveSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(_("No scene selected"), _("No scene selected, please select the scene that you would like to save"));
            return;
        }

        this.saveScene(node);
    },
    saveScene: function(node) {
        if ((node === null) || (!node.isLeaf())) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
    },
    renameSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(_("Error"), _("No scene selected, please select the scene that you want to rename"));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) ||
            (ids.groupId === null) || (ids.sceneNumber === null)) {
            Ext.Msg.alert(_("Error"), _("Could not determine scene from node"));
            return;
        }

        var sceneStore = dSS.data.Loader.getSceneStore();
        var recId = ids.zoneId + '-' + ids.groupId + '-' + ids.sceneNumber;

        var record = sceneStore.getById(recId);

        // this should never be the case, however if by any chance we missed
        // something I prefer having a messagebox here instead of some
        // random error
        if (record === undefined) {
            record = new sceneStore.recordType({
                                                    name : "",
                                                    scene: ids.sceneNumber,
                                                    group: ids.groupId,
                                                    zone: ids.zoneId
                                                }, recId);
            sceneStore.add([record]);
        }

        Ext.Msg.prompt(_("Rename scene"), _("New name for scene"),
                function(btn, text) {
                if (text !== record.get('name')) {
                    Ext.Ajax.request({
                        url: '/json/zone/sceneSetName',
                        disableCaching: true,
                        method: "GET",
                        params: {
                            id: ids.zoneId,
                            groupID: ids.groupId,
                            sceneNumber: ids.sceneNumber,
                            newName: text
                        },
                        success: function(result, request) {
                            try {
                                var jsonData = Ext.util.JSON.decode(result.responseText);
                                if (jsonData.ok) {
                                    record.set('name', text);
                                    sceneStore.commitChanges();
                                } else {
                                    Ext.MessageBox.alert(_("Error"), _("Could not rename scene"));
                                }
                            }
                            catch(err) {
                                Ext.MessageBox.alert(_("Error"), _("Could not rename scene") + ': ' + err);
                            }
                        },
                        failure: function(result, request) {
                            Ext.MessageBox.alert(_("Error"), _("Could not rename scene"));
                        }
                    });
                }
            },
            this, false, record.get('name'));
    }
});

Ext.reg('dssscenespanel', dSS.ScenesPanel);

