//= require <dSS/GetTextHelper>
//= require <dSS/data/ZoneStore>
//= require <dSS/data/Loader>
Ext.namespace('dSS');

dSS.ZoneView = Ext.extend(Ext.DataView, {
    initComponent: function() {

        var zoneTemplate = new Ext.XTemplate(
            '<tpl for=".">', 
                '<div class="zone-wrap {css}" id="zone-{id}">', 
                    '<span>', 
                        '{name} - {id}', 
                        '<tpl if="primary === true">', 
                            ' *', 
                        '</tpl>', 
                    '</span>', 
                '</div>', 
            '</tpl>', 
            '<div class="x-clear"></div>');

        Ext.apply(this, {
            store: dSS.data.Loader.getZoneStore(),
            tpl: zoneTemplate,
            singleSelect: true,
            multiSelect: false,
            layout: 'fit',
            style: "overflow: auto",
            overClass: 'x-view-over',
            itemSelector: 'div.zone-wrap',
            emptyText: _("No rooms to display")
        });

        dSS.ZoneView.superclass.initComponent.apply(this, arguments);

        dSS.data.Loader.getDeviceStore().on('load', this.updateNumDevicesHeader, this);

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('selectionchange', function(dv, nodes) {
            /*
				var l = nodes.length;
				var s = l != 1 ? 's' : '';
				this.findParentByType('dsszonepanel').setTitle(gt.ngettext("Zone", "n% Zones", l));
            */
            this.filterDevices();
            this.updateNumDevicesHeader();
        },
        this);

        this.on('render', function() {
            var zoneView = this;
            this.dropZone = new Ext.dd.DropZone(zoneView.getEl(), {
                ddGroup: "zoneDeviceDD",
                containerScroll: true,
                getTargetFromEvent: function(e) {
                    return e.getTarget(zoneView.itemSelector);
                },
                onNodeEnter: function(target, dd, e, data) {
                    Ext.fly(target).addClass('my-row-highlight-class');
                },
                onNodeOut: function(target, dd, e, data) {
                    Ext.fly(target).removeClass('my-row-highlight-class');
                },
                onNodeOver: function(target, dd, e, data) {
                    return Ext.dd.DropZone.prototype.dropAllowed;
                },
                onNodeDrop: function(target, dd, e, data) {
                    var record = zoneView.getRecord(target);
                    Ext.each(data.selections, function(device) {
                        var currentDevice = device;
                        Ext.Ajax.request({
                            url: '/json/structure/zoneAddDevice',
                            disableCaching: true,
                            method: "GET",
                            scope: zoneView,
                            params: {
                                deviceID: currentDevice.get('dsid'),
                                zone: record.get('id')
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {
                                        currentDevice.set("zoneId", record.data.id);
                                        currentDevice.store.commitChanges();
                                        this.filterDevices();
                                        this.updateNumDevicesHeader();
                                        this.findParentByType('tabpanel').scenesPanel.markDirty();
                                    }
                                }
                                catch(err) {
                                    Ext.MessageBox.alert(_("Error"), _("Could not move device ") + '"' + device.data.dsid + '": ' + err);
                                }
                            },
                            failure: function(result, request) {
                                Ext.MessageBox.alert(_("Error"), _("Could not move device ") + '"' + device.data.dsid + '"');
                            }
                        });
                    });
                    return true;
                }
            });
        },
        this);

        this.on('contextmenu', function(view, index, node, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var menuItem = new Ext.menu.Item({
                text: _("Rename room"),
                iconCls: 'editZoneAction',
                handler: this.findParentByType('dsszonepanel').editZone.createDelegate(this.findParentByType('dsszonepanel'), index, true)
            });
            this.contextMenu.add(menuItem);

            var menuItem = new Ext.menu.Item({
                text: _("Delete room"),
                iconCls: 'removeRoomIcon',
                handler: this.removeZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            var menuItem = new Ext.menu.Item({
                text: _("Blink"),
                iconCls: 'blinkActive',
                handler: this.blinkZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    filterDevices: function() {
        var selectedZones = this.getSelectedRecords();
        var store = dSS.data.Loader.getDeviceStore();

        if (selectedZones.length === 0) {
            store.clearFilter();
            return;
        }
        store.filterBy(function(record) {
            for (var i = 0; i < selectedZones.length; i++) {
                if (record.get('zoneId') === selectedZones[i].get('id')) {
                    return true;
                }
            }
            return false;
        });
    },

    updateNumDevicesHeader: function() {
        var count = dSS.data.Loader.getDeviceStore().getRange().length;
        var selectedZones = this.getSelectedRecords();
        var title = '';
        if (selectedZones.length === 0) {
            title = _("Devices in all rooms");
        } else {
            title = _("Devices in room") + " \"" + selectedZones[0].get('name') + " - " + selectedZones[0].get('id') + "\"";
        }
        this.findParentByType('dsszonebrowser').devicePanel.setTitle("<span class=\"x-panel-header-text\">" + title + "</span><span style=\"float: right;\" class=\"x-panel-header-text\">" + count + " " + gt.ngettext("Device", "n% Devices", count) + "</span>");
    },

    removeZone: function(item, event, index) {
        var record = this.getStore().getAt(index);
        if (record.get('primary') === true) {
            Ext.MessageBox.alert('Error', 'You cannot delete a primary room.');
            return;
        }
        var deviceStore = this.findParentByType('dsszonebrowser').devicePanel.getStore();
        if (deviceStore.query('zone', record.get('id')).getCount() > 0) {
            Ext.MessageBox.alert('Error', 'You cannot delete a non empty room.');
            return;
        }

        Ext.Ajax.request({
            url: '/json/structure/removeZone',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                zoneID: record.get('id')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        this.getStore().removeAt(index);
                    } else {
                        Ext.MessageBox.alert(_("Error"), _("Could not remove room") + " \"" + record.get('name') + "\"");
                    }
                } catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could not remove room") + " \"" + record.get('name') + "\"");
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could not remove room") + " \"" + record.get('name') + "\"");
            }
        });
    },

    blinkZone: function(item, event, index) {
        var record = this.getStore().getAt(index);

        Ext.Ajax.request({
            url: '/json/zone/blink',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                id: record.get('id')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.MessageBox.alert(_("Error"), _("Could not blink room") + " \"" + record.get('name') + "\"");
                    }
                } catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could not blink room") + " \"" + record.get('name') + "\"");
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could not blink room") + " \"" + record.get('name') + "\"");
            }
        });
    }
});

Ext.reg('dsszoneview', dSS.ZoneView);

