//= require <dSS/util/Util

Ext.namespace("dSS", "dSS.config", "dSS.devprops", "dSS.util");

dSS.devprops.OutputModeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(_("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;

        this.addEvents('formsubmitted', 'formloaded');

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            params: { class : 3, index : 0, dsid : config.dsid },
            items:
            [
                {
                    xtype: 'radiogroup',
                    ref: 'radio',
                    fieldLabel: 'Output mode',
                    columns: 1,
                    items:
                    [
                        {
                            boxLabel: _("switch"),
                            name: 'rb_outmode_' + config.dsid,
                            id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                            inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH,
                            checked: true
                        },
                        {
                            boxLabel: _("dimmer"),
                            name: 'rb_outmode_' + config.dsid,
                            id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER.toString(),
                            inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER
                        },
                        {
                            boxLabel: _("disabled"),
                            name: 'rb_outmode_' + config.dsid,
                            id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                            inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                        }
                    ]
                }
            ]

        });

        dSS.devprops.OutputModeForm.superclass.constructor.apply(this, arguments);
        // if masking is done in afterrender, the animation is not positioned
        // correctly; however we can not do the ajax call in afterlayout,
        // because it seems that afterlayout gets called twice
        this.on('afterlayout', function() {
                this.el.mask(_("Loading output mode settings") + "...",
                             'x-mask-loading');
            },
            this);

        this.on('afterrender', function() {
            this.AjaxRequest = Ext.Ajax.request(
                {
                    url: '/json/device/getConfig',
                    method: 'GET',
                    params:
                    {
                        class : 3,
                        index : 0,
                        dsid : this.dsid
                    },
                    scope: this,
                    success: function(response) {
                        this.AjaxRequest = null;
                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                response.responseText);
                            if (jsonData.ok !== true) {
                                this.el.unmask();
                                this.fireEvent('formloaded');
                                return;
                            }

                            // window closed during the reques
                            if (this.radio === undefined) {
                                this.el.unmask();
                                this.fireEvent('formloaded');
                                return;
                            }

                            var val = jsonData.result.value;
                            this.radio.setValue(val, val);
                            this.loadedValue = val;
                            this.el.unmask();
                            this.fireEvent('formloaded');

                        } catch (error) {
                            Ext.Msg.alert(_("Error"), error);
                            this.fireEvent('formloaded');
                        }
                    },
                    failure: function(result, request) {
                        this.AjaxRequest = null;
                        Ext.Msg.alert(_("Error"),
                             _("Could not load output mode settings"));
                        this.el.unmask();
                        this.fireEvent('formloaded');
                        // window closed during the reques
                        if (this.radio === undefined) {
                            return;
                        }
                        this.radio.markInvalid(
                             ("Current settings could not be loaded"));
                    }
                });
            },
            this);

        this.on('beforedestroy', function() {
                if (this.AjaxRequest !== null) {
                    Ext.Ajax.abort([this.AjaxRequest.tId]);
                }
            },
            this);

        this.on('submit', function() {
            var value = this.radio.getValue().getRawValue();

            if (value == this.loadedValue) {
                this.fireEvent('formsubmitted');
                return;
            }

            // this should never happen, but lets double check so we
            // do not attempt to write some random crap into the klemme
            if ((value !== dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString()) && 
                (value !== dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString()) && 
                (value !== dSS.config.deviceParams.OUTPUT_MODE_DIMMER.toString())) {
                Ext.Msg.alert(_("Error"), _("Attempted to set invalid value") +
                    ": " + value);
                this.fireEvent('formsubmitted');
                return;
            }

            this.el.mask(_("Saving output mode settings") + "...",
                           'x-mask-loading');

            this.AjaxRequest = Ext.Ajax.request(
                {
                    url: '/json/device/setConfig',
                    method: 'GET',
                    params:
                    {
                        class : 3,
                        index : 0,
                        dsid : this.dsid,
                        value : value
                    },
                    scope: this,
                    success: function(response) {
                        this.el.unmask();
                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                response.responseText);
                            if (jsonData.ok !== true) {
                                Ext.Msg.alert(_("Error"),
                                    _("Could not save output mode settings") +
                                    ": " + jsonData.message);
                            }

                            // this will close the main dialog
                            this.fireEvent('formsubmitted');
                        } catch (error) {
                            Ext.Msg.alert(_("Error"), error);
                        }
                    },
                    failure: function(result, request) {
                        Ext.Msg.alert(_("Error"),
                             _("Could not save output mode settings!"));
                        this.el.unmask();
                        // this will close the main dialog
                        this.fireEvent('formsubmitted');
                    }
                });
            },
            this
            );
    }
});

Ext.reg('dssdevoutputmodeform', dSS.devprops.OutputModeForm);

// each returned item *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
dSS.devprops.GetParameterItemsForDevice = function(deviceClass, deviceType,
                                                   deviceNumber, dsid) {
    var ret = [];
    if (dSS.config.deviceColors[deviceClass] === 'GE') {
        if (((dSS.config.deviceTypes[deviceType] === 'KM') &&
             (deviceNumber === 200)) ||
            ((dSS.config.deviceTypes[deviceType] === 'TKM') &&
             (deviceNumber === 210)) ||
            ((dSS.config.deviceTypes[deviceType] === 'SDM') &&
             (deviceNumber === 200))) {
                ret.push({
                            xtype: 'dssdevoutputmodeform',
                            dsid : dsid,
                            ref: 'outputmodeform'
                         });
        }
    }
    return ret;
};

// parameters:
//      device record
//      maskElemen

dSS.devprops.DevicePropertiesWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        if (config.device === undefined) {
            Ext.MessageBox.alert(_("Error"), ("Invalid device record"));
            return;
        }

        var devInfo = dSS.util.parseProductID(config.device.get('productID'));

        var deviceType = devInfo[0];
        var deviceNumber = devInfo[1];

        if (dSS.config.deviceTypes[deviceType] === undefined) {
            Ext.MessageBox.alert(_("Error"), ("Invalid device type"));
            return;
        }

        /*
        if (dSS.config.deviceColors[deviceNumber] === undefined) {
            Ext.MessageBox.alert(_("Error"), ("Invalid device number"));
            return;
        }
        */

        var fullDSID = config.device.get('dsid');
        var dsid = fullDSID;
        if (dsid.length > 8) {
            dsid = dsid.slice(dsid.length - 8);
        }
        var name = config.device.get('name');

        if ((name === undefined) || (name.length === 0)) {
            name = '<i>' + _("no name") + '</i>';
        }

        var devClass = dSS.util.parseFunctionID(config.device.get('functionID'))[0];
        var icon = dSS.config.deviceIconCls[devClass];
        if (icon === undefined) {
            icon = '';
        }

        var items =
            [
                {
                    xtype: 'panel',
                    padding: 5,
                    bodyCfg: {
                        cls: 'x-window-body'
                    },
                    bodyBorder: false,
                    border: false,
                    frame: false,
                    layout: 'table',
                    layoutConfig: {
                        columns: 2
                    },
                    items:
                    [
                        {
                            xtype: 'label',
                            ctCls: 'x-text-nowrap',
                            html: _("Device name") + ':&nbsp;'
                        },
                        {
                            xtype: 'label',
                            ctCls: 'x-text-align-left x-text-nowrap',
                            html: name
                        },
                        {
                            xtype: 'label',
                            ctCls: 'x-text-nowrap',
                            html: _("Device ID") + ':&nbsp;'
                        },
                        {
                            xtype: 'label',
                            ctCls: 'x-text-align-left x-text-nowrap',
                            html: dsid,
                            bodyBorder: false
                        }
                    ]
                }
            ];
        var forms = dSS.devprops.GetParameterItemsForDevice(devClass,
                                                            deviceType,
                                                            deviceNumber,
                                                            fullDSID);

        this.expectedChildEvents = forms.length;
        this.receivedSaveEvents = 0;
        this.receivedLoadEvents = 0;
        items = items.concat(forms);
        this.addEvents('formsubmitted', 'formloaded');

        this.saveButton = new Ext.Button({
            text: _("Save"),
            disabled: true,
            scope: this,
            handler: this.saveSettings.createDelegate(this)
        });

        this.cancelButton = {
            xtype: 'button',
            text: _("Cancel"),
            scope: this,
            handler: function() {
                this.maskElement.unmask();
                this.close();
            }
        };


        Ext.apply(this, {
            layout: 'table',
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            padding: 10,
            width: 400,
            resizable: false,
            iconCls: icon,
            title: _("Device properties"),
            items: items,
            buttons: [ this.saveButton, this.cancelButton ]
        });

        dSS.devprops.DevicePropertiesWindow.superclass.constructor.apply(this, arguments);

        this.maskElement.mask();

        this.on('beforedestroy',
            function(component) {
                this.maskElement.unmask();
            },
            this);

        this.on('formloaded',
            function() {
                this.receivedLoadEvents++;
                if (this.receivedLoadEvents >= this.expectedChildEvents) {
                    this.receivedLoadEvents = 0;
                    this.saveButton.enable();
                }
            },
            this);

        this.on('formsubmitted',
            function() {
                this.receivedSaveEvents++;
                if (this.receivedSaveEvents >= this.expectedChildEvents) {
                    this.maskElement.unmask();
                    this.close();
                }
            },
            this);
    },
    saveSettings: function() {
        this.el.mask();
        this.receivedSaveEvents = 0;
        this.items.each(function(item) { item.fireEvent('submit'); });
    }


});
