//= require <dSS/GetTextHelper>
//= require <dSS/data/DeviceStore>
//= require <dSS/data/Loader>
//= require <dSS/util/GridToSVWindow>
//= require <dSS/util/Util>
//= require <dSS/util/DefaultDialog>
//= requie  <dSS/config/Config>
//= require <dSS/scene/SceneWindowShadow>
//= require <dSS/scene/SceneWindowLight>
//= require <dSS/devprops/DeviceProperties>

Ext.namespace('dSS', 'dSS.grid');

dSS.grid.DevicePanel = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {
        var commentRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            metaData.css = 'commentIcon';

            var comment = record.get('comment');

            if ((comment !== undefined) && (comment.length > 0)) {
                return String.format('<div qtip="{0}">&nbsp;&nbsp;&nbsp;</div>',
                        comment);
            } else {
                return '<div>&nbsp;&nbsp;&nbsp;</div>';
            }

        };

        var dSIDdeviceRenderer= function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = dSS.util.shortDSID(record.get('dsid'));
            return String.format('<div qtip="{0}">{1}</div>',
                                  record.get('dsid'), shortId);
        };

        var dSIDmeterRenderer= function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = dSS.util.shortDSID(record.get('meterDSID'));
            return String.format('<div qtip="{0}">{1}</div>',
                                  record.get('meterDSID'), shortId);
        };

        var devTypeRenderer = function(value, metaData, record, rowIndex, colIndex, store) {

            var present = record.get('isPresent');

            var fid = dSS.util.parseFunctionID(record.get('functionID'));

            var retval = String.format('<img class="padding-img" src="{0}"/>', Ext.BLANK_IMAGE_URL);

            switch (fid[0]) {
            case dSS.config.groups.LIGHT:
                if (present === true) {
                    metaData.css = 'functionLight';
                } else {
                    metaData.css = 'functionLightDisabled';
                }
                break;
            case dSS.config.groups.SHADOW:
                if (present === true) {
                    metaData.css = 'functionShadow';
                } else {
                    metaData.css = 'functionShadowDisabled';
                }
                break;
            case dSS.config.groups.CLIMATE:
                metaData.css = 'functionClimate';
                break;
            case dSS.config.groups.AUDIO:
                metaData.css = 'functionAudio';
                break;
            case dSS.config.groups.VIDEO:
                metaData.css = 'functionVideo';
                break;
            case dSS.config.groups.SECURITY:
                metaData.css = 'functionSecurity';
                break;
            case dSS.config.groups.ACCESS:
                metaData.css = 'functionAccess';
                break;
            case dSS.config.groups.JOKER:
                metaData.css = 'functionJoker';
                break;
            default:
                retval = "";
                break;
            }

            return retval;
        };

        var deviceCols = [
        {
            header: "",
            width: 17,
            resizable: false,
            sortable: false,
            hideable: false,
            menuDisabled: true,
            fixed: true,
            dataIndex: 'functionID',
            renderer: devTypeRenderer,
            id: 'functionID'
        },
        {
            header: _("dSID"),
            tooltip: _("dSID"),
            width: 70,
            sortable: true,
            dataIndex: 'dsid',
            id: 'dsid',
            renderer: dSIDdeviceRenderer
        },
        {
            header: _("name"),
            tooltip: _("name"),
            width: 80,
            sortable: true,
            dataIndex: 'name',
            id: 'name'
        },
        {
            header: _("dSM Name"),
            tooltip: _("dSM Name"),
            width: 80,
            sortable: true,
            dataIndex: 'modulatorName',
            id: 'modulatorName'
        },
        {
            header: _("dSM dSID"),
            tooltip: _("dSM dSID"),
            width: 72,
            sortable: true,
            dataIndex: 'meterDSID',
            id: 'meterDSID',
            renderer:dSIDmeterRenderer
        },
        {
            header: _("zone id"),
            tooltip: _("zone id"),
            width: 35,
            sortable: true,
            dataIndex: 'zoneId',
            id: 'zoneId'
        },
        {
            header: _("Comment"),
            tooltip: _("Comment"),
            width: 20,
            resizable: false,
            sortable: false,
            dataIndex: 'comment',
            id: 'comment',
            renderer: commentRenderer
        },
        {
            header: _("first seen"),
            tooltip: _("first seen"),
            width: 60,
            sortable: true,
            dataIndex: 'firstSeen',
            xtype: 'datecolumn',
            format: 'Y-m-d\\&\\n\\b\\s\\p\\;H:m:i',
            id: 'firstSeen'
        },
        {
            header: _("last discovered"),
            tooltip: _("last discovered"),
            width: 60,
            sortable: true,
            dataIndex: 'lastDiscovered',
            xtype: 'datecolumn',
            format: 'Y-m-d\\&\\n\\b\\s\\p\\;H:m:i',
            id: 'lastDiscovered'
        },
        {
            header: _("HW info"),
            tooltip: _("Hardware info"),
            width: 50,
            sortable: true,
            id: 'HW',
            dataIndex: 'HW'
        }];

        Ext.apply(this, {
            store: dSS.data.Loader.getDeviceStore(),
            columns: deviceCols,
            ddGroup: "zoneDeviceDD",
            enableDragDrop: true,
            stripeRows: true,
            title: gt.ngettext("Device", 2),
            viewConfig: {
                forceFit: true,
                getRowClass: function(record, index) {
                    var c = record.get('isPresent');
                    if (c === false) {
                        return 'nonPresentDevice';
                    }
                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        dSS.grid.DevicePanel.superclass.initComponent.apply(this, arguments);

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('rowcontextmenu', function(grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();
            if (!sel.isSelected(rowIndex)) {
                sel.selectRow(rowIndex, false);
            }

            var showRemoveMenuItem = false;
            var presentInSelection = false;
            var selectedDeviceIsPresent = false;

            if (sel.getSelected().get("isPresent") === true) {
                selectedDeviceIsPresent = true;
            }

            var records = sel.getSelections();

            for (var i = 0; i < records.length; i++) {
                var record = records[i];

                // if at least one record can be removed - set the flag
                if (record.get('isPresent') === false) {
                    showRemoveMenuItem = true;
                } else {
                    presentInSelection = true;
                }

                if ((showRemoveMenuItem === true) && (presentInSelection === true)) {
                    break;
                }
            }

            if (presentInSelection === true) {
                var menuItem = new Ext.menu.Item({
                    text: _("Blink"),
                    iconCls: 'blinkActive',
                    handler: this.blinkDevice.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            if (selectedDeviceIsPresent === true) {
               var menuItem = new Ext.menu.Item({
                    text: _("Edit device values"),
                    iconCls: 'editValueToolbarIcon',
                    handler: this.showControlDeviceWindow.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: _("Edit device properties"),
                    iconCls: 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            var menuItem = new Ext.menu.Item({
                text: _("Rename device"),
                iconCls: 'editIcon',
                handler: this.editDevice.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            var menuItem = new Ext.menu.Item({
                text: _("Edit comment"),
                iconCls: 'commentIcon',
                handler: this.editComment.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            if (selectedDeviceIsPresent === true) {
                menuItem = new Ext.menu.Item({
                    text: _("Test transmission quality"),
                    iconCls: 'transmissionQuality',
                    handler: this.getTransmissionQuality.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            if (showRemoveMenuItem === true) {
                menuItem = new Ext.menu.Item({
                    text: _("Remove device"),
                    iconCls: 'removeDeviceIcon',
                    handler: this.removeDevice.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }


            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    buildTopToolbar: function() {
        return [
        {
            iconCls: 'blinkActive',
            handler: this.blinkDevice,
            tooltip: _("Blink device"),
            scope: this
        },
        {
            iconCls: 'editValueToolbarIcon',
            handler: this.showControlDeviceWindow,
            tooltip: _("Edit device values"),
            scope: this
        },
        {
            iconCls: 'devicePropertiesIcon',
            handler: this.showDevicePropertiesWindow,
            tooltip: _("Configure device properties"),
            scope: this
        },
        {
            iconCls: 'editDeviceToolbarIcon',
            handler: this.editDevice,
            tooltip: _("Rename device"),
            scope: this
        },
        {
            iconCls: 'commentIcon',
            handler: this.editComment,
            tooltip: _("Edit comment"),
            scope: this
        },
        {
            iconCls: 'transmissionQuality',
            handler: this.getTransmissionQuality,
            tooltip: _("Test transmission quality"),
            scope: this
        },
        {
            iconCls: 'copyToClipboardToolbarIcon',
            handler: this.showCopyWindow,
            tooltip: _("Show in copy to clipboard view"),
            scope: this
        },
        '->',
        {
            iconCls: 'removeDeviceToolbarIcon',
            handler: this.removeDevice,
            tooltip: _("Remove device"),
            scope: this
        }];
    },

    showControlDeviceWindow: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(_("Error"),
                    _("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        var fid = dSS.util.parseFunctionID(record.get('functionID'));
        switch (fid[0]) {
        case 1:
            var winl = new dSS.scene.SceneWindowLight({
                dsid: record.get('dsid'),
                name: record.get('name'),
                maskElement: this.findParentByType(Ext.Viewport).el
            });
            winl.show(this);
            break;
        case 2:
            var wins= new dSS.scene.SceneWindowShadow({
                dsid: record.get('dsid'),
                name: record.get('name'),
                maskElement: this.findParentByType(Ext.Viewport).el
            });
            wins.show(this);
            break;
        default:
            Ext.Msg.alert(_("Warning"), _("Control dialog for this device type is not yet available"));
            break;
        }
    },
    editDevice: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(_("Error"),
                    _("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        Ext.Msg.prompt(_("Rename device"), _("Name for") + ' ' + dSS.util.shortDSID(record.get('dsid')) + ': ', function(btn, text) {
            if (text !== record.get('name')) {
                Ext.Ajax.request({
                    url: '/json/device/setName',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        dsid: record.get('dsid'),
                        newName: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                                record.set('name', text);
                                record.commit();
                            } else {
                                Ext.MessageBox.alert(_("Error"), _("Could not rename device"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(_("Error"), _("Could not rename device"));
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(_("Error"), _("Could not rename device"));
                    }
                });
            }
        },
        this, false, record.get('name'));
    },

    getTransmissionQuality: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(_("Error"),
                    _("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(_("Error"), _("Device not present: can not request transmission quality from a non present device!"));
            return;
        }

        var name = record.get('name');
        var dsid = record.get('dsid');

        Ext.Ajax.request({
            scope: this,
            url: '/json/device/getTransmissionQuality',
            disableCaching: true,
            method: "GET",
            params: {
                dsid: dsid
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var title = _("Transmission Quality");

                        var up = jsonData.result.upstream * (100/dSS.config.TransmissionQuality.UPSTREAM_MAX);
                        var down = 100 - jsonData.result.downstream * (100/dSS.config.TransmissionQuality.DOWNSTREAM_MAX);
                        up = Math.round(up);
                        down = Math.round(down);

                        var win = new dSS.util.DefaultDialogWindow({
                            title: title,
                            name: name,
                            dsid: dsid,
                            extraItems:
                            [
                                {
                                    xtype: 'dssutildialogcontentpanel',
                                    strings:
                                    [
                                        _("Upstream communication")+':&nbsp;',
                                        up + '%',
                                        _("Downstream communication")+':&nbsp;',
                                        down + '%'
                                    ]
                                }
                            ],
                            maskElement: this.findParentByType(Ext.Viewport).el
                        });
                        win.show();

                    } else {
                        Ext.MessageBox.alert(_("Error"),
                                _("Could not get transmission quality: "),
                                  jsonData.message);
                    }
                }
                catch(err) {
                    Ext.Msg.alert(_("Error"),
                                  _("Could not get transmission quality") +
                                  ': ' + err);
                }
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if ((!jsonData.ok) && (jsonData.message.length > 0)) {
                        message = ': ' + jsonData.message;
                    }
                }
                catch(err) {}
                Ext.MessageBox.alert(_("Error"),
                        _("Could not get transmission quality") + message);
            }
        });
    },

    removeDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(_("Error"),
                _("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        for (var i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === true) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length > 0) {
            this.deleteDevice(notPresent);
        } else {
            if (skippedPresent === true) {
                Ext.MessageBox.alert(_("Error"), _("Present devices can not be removed!"));
            }
        }
    },

    deleteDevice: function(records) {
        var store = this.getStore();
        if (records.length < 1) {
            this.el.unmask();
            return;
        }

        var record = records.shift();

        this.el.mask(_("Deleting device") + ": " + record.get("dsid"));
        Ext.Ajax.request({
            url: '/json/structure/removeDevice',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                deviceID: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        store.remove(record);
                        this.deleteDevice(records);
                    } else {
                        this.el.unmask();
                        Ext.MessageBox.alert(_("Could not remove"), jsonData.message);
                    }
                }
                catch(err) {
                    this.el.unmask();
                    Ext.MessageBox.alert(_("Error"), _("Could send remove device command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                this.el.unmask();
                Ext.MessageBox.alert(_("Error"), _("Could send remove device command to dSS"));
            }
        });
    },

    sendBlinkRequest: function(record) {
        record.set('blink', true);
        var unblink = function(record) {
            record.set('blink', false);
        };

        Ext.Ajax.request({
            url: '/json/device/blink',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsid: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.MessageBox.alert(_("Could not blink"), jsonData.message);
                    }
                } catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could not send blink device command to dSS") + ": " + err);
                }
                unblink.defer(2000, this, [record]);
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could not send blink device command to dSS"));
                unblink.defer(2000, this, [record]);
            }
        });
    },

    blinkDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(_("Error"),
                _("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();
        for (var i = 0; i < records.length; i++) {
            if (records[i].get('isPresent') === false) {
                continue;
            }

            this.sendBlinkRequest(records[i]);
        }
    },

    showDevicePropertiesWindow: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(_("Error"),
                    _("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(_("Error"), ("Device not present: The selected device was not found and can not be configured!"));
            return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            device : record,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },

    showCopyWindow: function() {
        var tsvwindow = new dSS.util.GridToSVWindow({
            ignore: [ 'functionID' ],
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 480,
            separator: "\t"
        });
        tsvwindow.show(this);
    },

    editComment: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(_("Error"),
                    _("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        Ext.Msg.prompt(
            _("Edit comment"), 
            _("Comment for") + ' ' + dSS.util.shortDSID(record.get('dsid')) + ': ', 
            function(btn, text) {
            if (text !== record.get('comment')) {
                Ext.Ajax.request({
                    url: '/json/property/setString',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        path:   '/apartment/zones/zone0/' +
                                record.get('dsid') + '/comment',
                        value: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                        result.responseText);
                            if (jsonData.ok) {
                                Ext.Ajax.request({
                                    url: '/json/property/setFlag',
                                    disableCaching: true,
                                    method: "GET",
                                    params: {
                                        flag: 'ARCHIVE',
                                        path:   '/apartment/zones/zone0/' +
                                                record.get('dsid') + '/comment',
                                        value: true
                                    },
                                    success: function(result, request) {
                                        try {
                                            var jsonData = Ext.util.JSON.decode(
                                                result.responseText);
                                            if (jsonData.ok) {
                                                record.set('comment', text);
                                                record.commit();
                                            }
                                        } catch(err) {
                                            Ext.MessageBox.alert(_("Error"), 
                                            _("Could not save comment on dSS") +
                                              ": " + err);
                                        }
                                    },
                                    failure: function(result, request) {
                                        Ext.MessageBox.alert(_("Error"), 
                                            _("Could not save comment on dSS"));
                                    }
                                });
                            } else {
                                Ext.MessageBox.alert(_("Error"), 
                                                    _("Could not add comment"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(_("Error"), 
                                     _("Could not add comment") + ": " + err);
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(_("Error"), 
                                _("Could not send command to dSS"));
                    }
                });
            }
        },
        this, true, record.get('comment'));
    }
});

Ext.reg('dssdevicepanel', dSS.grid.DevicePanel);

