//= require <dSS/util/Util>

Ext.namespace("dSS", "dSS.config", "dSS.util");

dSS.util.DefaultDialogContentPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        var items = [];

        if (config.strings !== undefined)
        {
            for (var i = 0; i < config.strings.length; i++) {
                var item = {
                    xtype: 'label',
                    html: config.strings[i],
                    cls: 'x-text-align-left x-text-nowrap'
                };
                items.push(item);
             }
        }
        Ext.apply(this, {
            xtype: 'panel',
            padding: 5,
            layout: 'table',
            layoutConfig: {
                columns: 2
            },
            items: items
        });

        dSS.util.DefaultDialogContentPanel.superclass.constructor.apply(this, arguments);
    }
});

Ext.reg('dssutildialogcontentpanel', dSS.util.DefaultDialogContentPanel);

dSS.util.DefaultDialogWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        var cancelButtonText = _("Close");
        
        if (config.cancelButtonText !== undefined) {
            cancelButtonText = config.cancelButtonText;
        }

        var dsid = dSS.util.shortDSID(config.dsid);
        var name = dSS.util.getName(config.name);

        var items =
            [
                {
                    xtype: 'panel',
                    padding: 5,
                    bodyCfg: {
                        cls: 'x-window-body'
                    },
                    bodyBorder: false,
                    border: false,
                    frame: false,
                    layout: 'table',
                    layoutConfig: {
                        columns: 2
                    },
                    items:
                    [
                        {
                            xtype: 'label',
                            ctCls: 'x-text-nowrap',
                            html: _("Device name") + ": " + '&nbsp;'
                        },
                        {
                            xtype: 'label',
                            ctCls: 'x-text-align-left x-text-nowrap',
                            html: name
                        },
                        {
                            xtype: 'label',
                            ctCls: 'x-text-nowrap',
                            html: _("Device ID") + ": " + '&nbsp;'
                        },
                        {
                            xtype: 'label',
                            ctCls: 'x-text-align-left x-text-nowrap',
                            html: dsid,
                            bodyBorder: false
                        }
                    ]
                }
            ];

        if ((config.extraItems !== undefined) && (config.extraItems.length>0)) {
            items = items.concat(config.extraItems);
        }

        this.cancelButton = {
            xtype: 'button',
            text: cancelButtonText,
            scope: this,
            handler: function() {
                this.maskElement.unmask();
                this.close();
            }
        };

        var buttons = [ this.cancelButton ];

        if ((config.buttons !== undefined) && (config.buttons > 0)) {
            buttons = buttons.concat(config.buttons);
        }

        Ext.apply(this, {
            layout: 'table',
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            padding: 10,
            width: 400,
            resizable: false,
            title: config.title,
            items: items,
            buttons: buttons
        });

        dSS.util.DefaultDialogWindow.superclass.constructor.apply(this, arguments);

        this.maskElement.mask();

        this.on('beforedestroy',
            function(component) {
                this.maskElement.unmask();
            },
            this);
    }
});
