//= require <dSS/GetTextHelper>
//= require <dSS/data/ZoneStore>
//= require <dSS/ZoneBrowser>
//= require <dSS/dSMOverviewPanel>
//= require <dSS/SystemPanel>
//= require <dSS/ScenesPanel>
//= require <dSS/data/Loader>
//= require <dSS/GlossaryPanel>
//= require <dSS/util/TimeInfo>
//= require <dSS/config/Config>
Ext.namespace("dSS", "dSS.config");

dSS.initUI = function() {
    Ext.get('start').remove();
	dSS.config.dSSTimeDiff = 0;

    Ext.QuickTips.init();

    var params = Ext.urlDecode(window.location.search.substring(1));

    var active = 0;

    var tabs = [
        {
            title: gt.ngettext("Zone", "n% Zones", 2),
            xtype: 'dsszonebrowser',
            ref: 'zoneBrowser'
        },
        {
            title: gt.ngettext("Scene", 2),
            xtype: 'dssscenespanel',
            ref: 'scenesPanel'
        },
        {
            title: gt.ngettext("Circuit", "n% Circuits", 2),
            xtype: 'dsmoverviewpanel',
            ref: 'dSMOverviewPanel'
        },
        {
            title: _("System Properties"),
            xtype: 'dsssystempanel',
            ref: 'systemPanel'
        },
        {
            title: _("Glossary"),
            xtype: 'dssglossarypanel',
            ref: 'glossaryPanel'
        }
    ];

    if (params.tab === 'zones') {
        active = 0;
    } else if (params.tab === 'scenes') {
        active = 1;
    } else if (params.tab === 'circuits') {
        active = 2;
    } else if (params.tab === 'sysprops') {
        active = 3;
    } else if (params.tab === 'glossary') {
        active = 4;
    }

    if (params.dss11 === "1") {
        tabs.unshift(
            {
                title: _("dSS11 Setup"),
                xtype: 'panel',
                cls: 'x-margined-green-text',
                html: _("Loading") + "...",
                autoEl: 'div',
                listeners: {
                    activate: function() {
                        window.location = window.location.protocol + '//' +
                            window.location.host;
                    }
                }
            }
        );
        active++;
    }

    var viewport = new Ext.Viewport({
        layout: 'border',
        items: [
        {
            xtype: 'box',
            region: 'north',
            height: 80,
            // give north and south regions a height
            autoEl: {
                tag: 'div',
                html: '<a href="http://digitalstrom.org/" alt="' + _("visit digitalSTROM.org") + '"><img id="logo" src="images/ds_logo.gif" alt="dS-Logo"></a>' + '<h1><a href="http://digitalstrom.org/" alt="visit digitalSTROM.org">' + _("digitalSTROM Setup") + '</a></h1>' + '<p><span id= \'dssVersion\'>-</span>, <span id=\'dssTime\'>-</span></p>'
            }
        },
        {
            region: 'center',
            xtype: 'tabpanel',
            activeTab: active,
            items: tabs
        }]
    });

    Ext.Ajax.request({
        url: "/json/system/version",
        disableCaching: true,
        success: function(response, options) {
            try {
                var data = Ext.decode(response.responseText);
                if (data.ok === false) {
                    throw new Ext.Error(_("Could not get dSS version"), data.error);
                }
                Ext.get("dssVersion").update(data.result.version);

            } catch(error) {}
        },
        failure: function(response, options) {}
    });

    Ext.Ajax.request({
        url: "/json/system/time",
        disableCaching: true,
        scope: this,
        success: function(response, options) {
            try {
                var data = Ext.decode(response.responseText);
                if (data.ok === false) {
                    throw new Ext.Error(_("Could not get dSS time"), data.error);
                }
                dSS.config.dSSTimeDiff = (Date.now() - (parseInt(data.result.time, 10) * 1000));
                dSS.time.update();

            } catch(error) {}
        },
        failure: function(response, options) {}
    });

    Ext.Ajax.request({
        url: "/json/property/getChildren",
        params: {
            path: "/system/js/settings"
        },
        scope: this,
        method: "GET",
        disableCaching: true,
        success: function(response, options) {
            try {
                var jsonData = Ext.util.JSON.decode(response.responseText);
                if (jsonData.ok) {
                    for (i = 0; i < jsonData.result.length; i++) {
                        /*
                            dSS.config.settings[jsonData.result[i].name] = jsonData.result[i];
                     */
                        getLogFileProperties(jsonData.result[i]);
                    }
                }
            } catch(err) {}
        },
        failure: function(response, options) {
            Ext.MessageBox.alert(_("Error"), _("Could send getChildren command to dSS"));
        }
    });

    function getLogFileProperties(res) {
        Ext.Ajax.request({
            url: "/json/property/getString",
            params: {
                path: "/system/js/settings/" + res.name + "/logfile"
            },
            scope: this,
            method: "GET",
            disableCaching: true,
            success: function(response, options) {
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok) {
                        dSS.config.settings[res.name] = {};
                        dSS.config.settings[res.name].logfile = jsonData.result.value;
                    }
                } catch(err) {}
            },
            failure: function(response, options) {
                Ext.MessageBox.alert(_("Error"), _("Could send getChildren command to dSS"));
            }
        });
    }
};

Ext.onReady(function() {
    Ext.Ajax.request({
        url: '/json/system/login',
        params: {
            user: 'dssadmin',
            password: 'dssadmin'
        },
        scope: this,
        method: "GET",
        disableCaching: true,
        success: function(response, options) {
            try {
                var jsonData = Ext.util.JSON.decode(response.responseText);
                if (jsonData.ok) {
                    dSS.initUI();
                } else {
                    Ext.Msg.alert(_("Error"), _("Could not log in") + ': ' +
                                  jsonData.message);
                }
            } catch(err) {
                Ext.Msg.alert(_("Error"), _("Could not log in") + ': ' + err);

            }
        },
        failure: function(response, options) {
            Ext.Msg.alert(_("Error"), _("Could not send login command to dSS"));
        }
    });
});

